'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _interpret = require('interpret');

var _interpret2 = _interopRequireDefault(_interpret);

var _rechoir = require('rechoir');

var _rechoir2 = _interopRequireDefault(_rechoir);

var _findUp = require('find-up');

var _findUp2 = _interopRequireDefault(_findUp);

var _path = require('path');

var _requireReload = require('require-reload');

var _requireReload2 = _interopRequireDefault(_requireReload);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var reload = (0, _requireReload2.default)(require);
var extensions = _interpret2.default.extensions;


var resolveFile = function resolveFile(filepath) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
	var _options$cwd = options.cwd,
	    cwd = _options$cwd === undefined ? process.cwd() : _options$cwd,
	    _options$useLoader = options.useLoader,
	    useLoader = _options$useLoader === undefined ? true : _options$useLoader,
	    _options$useFindUp = options.useFindUp,
	    useFindUp = _options$useFindUp === undefined ? true : _options$useFindUp,
	    _options$resolvers = options.resolvers,
	    resolvers = _options$resolvers === undefined ? [] : _options$resolvers,
	    _options$exts = options.exts,
	    exts = _options$exts === undefined ? [] : _options$exts;


	var uniqueResolvers = Array.from(new Set([cwd].concat(_toConsumableArray(resolvers))));
	var resolverPaths = uniqueResolvers.map(function (path) {
		return (0, _path.resolve)(cwd, path);
	});

	var safelyResolve = function safelyResolve(path) {
		try {
			return require.resolve(path);
		} catch (err) {
			return false;
		}
	};

	var requireResolve = function requireResolve(path) {
		if (resolvers.length) {
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = resolvers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var resolver = _step.value;

					var joinedPath = (0, _path.join)(resolver, path);
					if (!joinedPath.startsWith(_path.sep)) {
						joinedPath = '.' + _path.sep + joinedPath;
					}
					var res = safelyResolve(joinedPath);
					if (res) return res;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
		return safelyResolve(path);
	};

	var resolved = requireResolve(filepath);

	if (resolved) {
		return resolved;
	}

	var paths = [];

	if (useLoader) {
		(function () {
			var sortedExts = Object.keys(extensions).filter(function (ext) {
				return !exts.length || exts.indexOf(ext) > -1;
			}).sort(function (a, b) {
				return a === '.js' ? -1 : b === '.js' ? 1 : a.length - b.length;
			});
			var fileExt = (0, _path.extname)(filepath);
			var includesExt = function includesExt() {
				return sortedExts.reverse().some(function (ext) {
					return filepath.endsWith(ext);
				});
			};
			if (fileExt && includesExt()) {
				paths.push(filepath);
			} else {
				paths.push(filepath);
				sortedExts.forEach(function (ext) {
					paths.push(filepath + ext);
				});
			}
		})();
	} else {
		paths.push(filepath);
	}

	var fullPaths = [];
	resolverPaths.forEach(function (dir) {
		paths.forEach(function (path) {
			fullPaths.push((0, _path.resolve)(cwd, dir, path));
		});
	});

	var finalPath = fullPaths.find(requireResolve);

	if (!finalPath && useFindUp) {
		finalPath = _findUp2.default.sync(paths, { cwd: cwd });
	}

	if (!finalPath) {
		var error = new Error('Module "' + filepath + '" NOT found.');
		error.errno = 'ENOENT';
		error.cwd = cwd;
		error.paths = fullPaths;
		throw error;
	}

	return finalPath;
};

var importFile = function importFile(filepath) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
	var _options$useLoader2 = options.useLoader,
	    useLoader = _options$useLoader2 === undefined ? true : _options$useLoader2,
	    _options$useCache = options.useCache,
	    useCache = _options$useCache === undefined ? true : _options$useCache,
	    _options$useESDefault = options.useESDefault,
	    useESDefault = _options$useESDefault === undefined ? true : _options$useESDefault;

	var finalPath = resolveFile(filepath, options);
	try {
		useLoader && _rechoir2.default.prepare(extensions, finalPath);
	} catch (err) {
		/* noop */
	}
	var module = useCache ? require(finalPath) : reload(finalPath);
	var shouldUseDefault = useESDefault && module.__esModule && module.default;
	return shouldUseDefault ? module.default : module;
};

importFile.resolve = resolveFile;

exports.default = importFile;